/* !! This file was automatically generated by NEWVERS !! */

/* for logfiles, etc. */
char SMake_Version[] =
    "1.2b1 (04-05-1997)";

/* interactive 'hello' string to identify us to the user */
char SMake_Hello[] = 
    "This is SMake Version 1.2b1 (04-05-1997)";

/* a GNU --version output */
char SMake_GNUVersion[] =
    "SMake Version 1.2b1";

/* a UNIX what(1) id string */
char SMake_WhatID[] =
    "@(#)SMake Version 1.2b1 (04-05-1997)";

/* a RCS ident(1) id string */
char SMake_RCSIdentID[] =
    "$Id: SMake 1.2b1 04-05-1997 $";

/* a WWW id string */
char SMake_WebID[] =
    "SMake/1.2b1";

